@@ -89,8 +89,6 @@ enum {
 };
 
 /* Prototypes */
-static void gnome_font_picker_class_init        (GnomeFontPickerClass *class);
-static void gnome_font_picker_instance_init	(GnomeFontPicker      *cp);
 static void gnome_font_picker_destroy		(GtkObject            *object);
 static void gnome_font_picker_finalize		(GObject              *object);
 static void gnome_font_picker_get_property	(GObject              *object,
@@ -128,7 +126,7 @@ static void gnome_font_picker_update_font_info	(GnomeFontPicker     *gfs);
 static guint font_picker_signals[LAST_SIGNAL] = { 0 };
 
 GNOME_CLASS_BOILERPLATE (GnomeFontPicker, gnome_font_picker,
-			 GtkButton, gtk_button, GTK_TYPE_BUTTON)
+			 GtkButton, GTK_TYPE_BUTTON)
 
 static void
 gnome_font_picker_class_init (GnomeFontPickerClass *class)
@@ -286,7 +284,7 @@ gnome_font_picker_destroy (GtkObject *object)
 	    gfp->_priv->font_dialog = NULL;
     }
 
-    GNOME_CALL_PARENT_HANDLER (GTK_OBJECT_CLASS, destroy, (object));
+    GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
     
 } /* gnome_font_picker_destroy */
 
@@ -316,7 +314,7 @@ gnome_font_picker_finalize (GObject *object)
     g_free(gfp->_priv);
     gfp->_priv = NULL;
 
-    GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, finalize, (object));
+    GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
     
 } /* gnome_font_picker_finalize */
 
@@ -761,9 +759,10 @@ gboolean   gnome_font_picker_set_font_name    (GnomeFontPicker *gfp,
     g_return_val_if_fail (GNOME_IS_FONT_PICKER (gfp), FALSE);
     g_return_val_if_fail (fontname != NULL, FALSE);
 
-    /* g_free handles NULL */
-    g_free(gfp->_priv->font_name);
-    gfp->_priv->font_name = g_strdup(fontname);
+    if (gfp->_priv->font_name != fontname) {
+	    g_free(gfp->_priv->font_name);
+	    gfp->_priv->font_name = g_strdup (fontname);
+    }
     
     if (gfp->_priv->mode == GNOME_FONT_PICKER_MODE_FONT_INFO)
 	gnome_font_picker_update_font_info(gfp);
@@ -820,9 +819,10 @@ void	   gnome_font_picker_set_preview_text (GnomeFontPicker *gfp,
     g_return_if_fail (GNOME_IS_FONT_PICKER (gfp));
     g_return_if_fail (text != NULL);
 
-    /* g_free handles NULL */
-    g_free(gfp->_priv->preview_text);
-    gfp->_priv->preview_text = g_strdup(text);
+    if (gfp->_priv->preview_text != text) {
+	    g_free(gfp->_priv->preview_text);
+	    gfp->_priv->preview_text = g_strdup (text);
+    }
 
     if (gfp->_priv->font_dialog)
         gtk_font_selection_dialog_set_preview_text(GTK_FONT_SELECTION_DIALOG(gfp->_priv->font_dialog), gfp->_priv->preview_text);
@@ -925,6 +925,7 @@ gnome_font_picker_dialog_cancel_clicked(GtkWidget *widget,
 
     gtk_widget_hide(gfp->_priv->font_dialog);
 
+    /* FIXME: is this necessary? bug #67587 */
     /* Restore old values */
     gnome_font_picker_set_font_name(gfp,gfp->_priv->font_name);
     gnome_font_picker_set_preview_text(gfp,gfp->_priv->preview_text);
@@ -939,6 +940,7 @@ gnome_font_picker_dialog_delete_event(GtkWidget *widget, GdkEventAny *ev,
 
     gfp = GNOME_FONT_PICKER (data);
     
+    /* FIXME: is this necessary? bug #67587 */
     /* Here we restore old values */
     gnome_font_picker_set_font_name(gfp,gfp->_priv->font_name);
     gnome_font_picker_set_preview_text(gfp,gfp->_priv->preview_text);
